#!/usr/bin/env python3

import os
import time
import sys
import requests
import json
import re

from lxml import etree

headers = {
    'User-Agent': "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.163 Safari/537.36"
}

def usage():
    print("mangadex.py command")
    print("commands:")
    print("  download")
    print("  list")
    exit(1)

def usage_list():
    print("mangadex.py list <manga-id>")
    exit(1)

def usage_download():
    print("mangadex.py download <chapter-id> <download_dir>")
    print("examples:")
    print("  mangadex.py download \"4e4a1ed8-d4a0-4096-86db-ca5e3fc42c5d\" /home/user/Manga/MangaName")
    print("")
    print("Note: The manga directory has to exist.")
    exit(1)

if len(sys.argv) < 2:
    usage()

def download_file(url, save_path):
    with requests.get(url, headers=headers, stream=True, timeout=30) as response:
        if not response.ok:
            return False
        with open(save_path, "wb") as file:
            for chunk in response.iter_content(chunk_size=8192):
                if chunk:
                    file.write(chunk)
    return True

def chapter_sort_func(chapter_data):
    return chapter_data[1].get("timestamp", 0)

def chapter_title_extract_number(chapter_title):
    result = re.search("Ch. ([0-9]+)", chapter_title)
    if result and len(result.groups()) > 0:
        return result.groups()[0]

def sort_chapters(chapter):
    return chapter["chapter"]

def list_chapters(url, chapter_list_input):
    manga_id = url
    seen_titles = set()
    seen_chapter_numbers = set()
    for item in chapter_list_input:
        title = item.get("title")
        if title and len(title) > 0:
            seen_titles.add(title.lower().replace(" ", "").replace("/", "_"))
            chapter_number = chapter_title_extract_number(title)
            if chapter_number:
                seen_chapter_numbers.add(chapter_number)

    seen_urls = set()
    for item in chapter_list_input:
        chapter_url = item.get("url")
        if chapter_url and len(chapter_url) > 0:
            seen_urls.add(chapter_url)

    lang = "en"

    i = 0
    prev_chapter_number = ""
    chapters = []
    finished = False
    while not finished:
        url = "https://api.mangadex.org/manga/" + manga_id + "/feed?&limit=100&offset=%d&translatedLanguage[]=%s&order[chapter]=desc" % (i * 100, lang)
        response = requests.get(url, headers=headers, timeout=30)
        response.raise_for_status()

        if len(response.text) == 0:
            break

        json_response = response.json()

        if json_response["result"] != "ok":
            print("Got result != ok")
            exit(1)

        data_list = json_response["data"]
        if len(data_list) == 0:
            break

        for data in data_list:
            id = data["id"]
            attributes = data["attributes"]

            external_url = attributes.get("externalUrl", "")
            if external_url:
                continue

            if attributes["translatedLanguage"] != lang:
                continue

            chapter_number_str = attributes["chapter"]
            if chapter_number_str == prev_chapter_number:
                continue
            prev_chapter_number = chapter_number_str

            title = "Ch. " + chapter_number_str
            title_item = attributes.get("title")
            if title_item:
                title += " - " + title_item
            title = title.replace("/", "_")

            if title.lower().replace(" ", "") in seen_titles or id in seen_urls:
                finished = True
                break

            if chapter_number_str in seen_chapter_numbers:
                finished = True
                break

            chapters.append({ "name": title, "url": id })
        i += 1

    print(json.dumps(chapters))

def get_base_url_for_chapter(chapter_id):
    response = requests.get("https://api.mangadex.org/at-home/server/" + chapter_id, headers=headers, timeout=30)
    response.raise_for_status()
    return response.json()["baseUrl"]

def download_chapter(url, download_dir):
    response = requests.get("https://api.mangadex.org/at-home/server/" + url, headers=headers, timeout=30)
    response.raise_for_status()

    image_urls = []
    json_response = response.json()
    if json_response["result"] != "ok":
        print("Got item with result != ok")
        exit(1)

    base_url = json_response["baseUrl"]
    chapter = json_response["chapter"]
    data = chapter["data"]
    hash = chapter["hash"]
    image_names = chapter["data"]
    for image_name in image_names:
        image_urls.append(base_url + "/data/" + hash + "/" + image_name)

    in_progress_filepath = os.path.join(download_dir, ".in_progress")
    with open(in_progress_filepath, "w") as file:
        file.write(url)

    img_number = 1
    for image_url in image_urls:
        ext = image_url[image_url.rfind("."):]
        image_name = str(img_number) + ext
        image_path = os.path.join(download_dir, image_name)
        print("Downloading {} to {}".format(image_url, image_path))
        if not download_file(image_url, image_path):
            print("Failed to download image: %s" % image_url)
            os.remove(in_progress_filepath)
            exit(2)
        img_number += 1

    if img_number == 1:
        print("Failed to find images for chapter")
        os.remove(in_progress_filepath)
        exit(2)

    with open(os.path.join(download_dir, ".finished"), "w") as file:
        file.write("1")

    os.remove(in_progress_filepath)

command = sys.argv[1]
if command == "list":
    if len(sys.argv) < 3:
        usage_list()
    
    url = sys.argv[2]
    chapter_list_input = sys.stdin.read()
    if len(chapter_list_input) == 0:
        chapter_list_input = []
    else:
        chapter_list_input = json.loads(chapter_list_input)
    list_chapters(url, chapter_list_input)
elif command == "download":
    if len(sys.argv) < 4:
        usage_download()
    url = sys.argv[2]
    download_dir = sys.argv[3]
    download_chapter(url, download_dir)
else:
    usage()
